from datetime import timedelta

from django.utils import timezone

from germanium.decorators import data_consumer
from germanium.test_cases.default import GermaniumTestCase
from germanium.tools import assert_equal, assert_true, assert_false

from auth_token.models import Token
from auth_token.config import settings

from .base import BaseTestCaseMixin


__all__ = (
    'TokenTestCase',
)


class TokenTestCase(BaseTestCaseMixin, GermaniumTestCase):

    @data_consumer('create_user')
    def test_should_return_proper_string_format_for_expiration(self, user):
        expired_token = Token.objects.create(user=user, ip='127.0.0.1')
        Token.objects.filter(pk=expired_token.pk).update(
            last_access=timezone.now() - timedelta(seconds=settings.MAX_TOKEN_AGE))
        expired_token = Token.objects.get(pk=expired_token.pk)
        assert_equal('00:00:00', Token.objects.get(pk=expired_token.pk).str_time_to_expiration)

        non_expired_token = Token.objects.create(user=user, ip='127.0.0.1')
        assert_equal('0:59:59', non_expired_token.str_time_to_expiration.split('.')[0])



from germanium.tools.rest import assert_valid_JSON_response

from is_core.config import settings as is_core_settings

from auth_token.config import settings
from auth_token.utils import header_name_to_django, create_auth_header_value


class RESTAuthMixin:

    def authorize(self, username, password):
        resp = self.post(
            is_core_settings.LOGIN_API_URL,
            data={is_core_settings.USERNAME: username, is_core_settings.PASSWORD: password}
        )
        assert_valid_JSON_response(resp, 'REST authorization fail: %s' % resp)
        self.default_headers[header_name_to_django(settings.HEADER_NAME)] = (
            create_auth_header_value(self.deserialize(resp).get('token'))
        )



from datetime import timedelta

from io import StringIO

from django.core.management import call_command
from django.utils import timezone

from germanium.decorators import data_consumer
from germanium.test_cases.default import GermaniumTestCase
from germanium.tools import assert_equal

from auth_token.models import Token
from auth_token.config import settings

from .base import BaseTestCaseMixin


__all__ = (
   'CleanTokensCommandTestCase',
)


class CleanTokensCommandTestCase(BaseTestCaseMixin, GermaniumTestCase):

    @data_consumer('create_user')
    def test_clean_tokens_remove_only_old_tokens(self, user):
        expired_tokens = [Token.objects.create(user=user, ip='127.0.0.1') for _ in range(10)]
        not_expired_tokens = [Token.objects.create(user=user, ip='127.0.0.1') for _ in
                              range(settings.COUNT_USER_PRESERVED_TOKENS - 5)]
        Token.objects.filter(pk__in=[token.pk for token in expired_tokens]).update(
            last_access=timezone.now() - timedelta(seconds=settings.MAX_TOKEN_AGE))
        call_command('clean_tokens', stdout=StringIO(), stderr=StringIO())
        assert_equal(Token.objects.filter(pk__in=[token.pk for token in not_expired_tokens]).count(),
                     settings.COUNT_USER_PRESERVED_TOKENS - 5)
        assert_equal(Token.objects.filter(pk__in=[token.pk for token in expired_tokens]).count(), 5)



from django.contrib.auth.views import LoginView, LogoutView
from django.utils.decorators import method_decorator
from django.http import HttpResponseRedirect
from django.views.decorators.cache import never_cache

from auth_token.contrib.common.forms import TokenAuthenticationForm
from auth_token.utils import login, logout


class TokenLoginView(LoginView):

    form_class = TokenAuthenticationForm
    allowed_cookie = True
    allowed_header = False

    def _login(self, user, expiration, form):
        login(
            self.request, user, expiration, allowed_cookie=self.allowed_cookie, allowed_header=self.allowed_header
        )

    def form_valid(self, form):
        """
        The user has provided valid credentials (this was checked in AuthenticationForm.is_valid()). So now we
        can check the test cookie stuff and log him in.
        """
        self._login(form.get_user(), not form.is_permanent(), form)
        return HttpResponseRedirect(self.get_success_url())


class TokenLogoutView(LogoutView):

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        logout(request)
        next_page = self.get_next_page()
        if next_page:
            # Redirect to this page until the session has been cleared.
            return HttpResponseRedirect(next_page)
        return super(LogoutView, self).dispatch(request, *args, **kwargs)


class InputLogMixin:

    def log_successful_request(self):
        pass

    def log_unsuccessful_request(self):
        pass



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-05-09 12:01
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('auth_token', '0007_auto_20190313_1528'),
    ]

    operations = [
        migrations.AlterField(
            model_name='devicekey',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True,
                                       verbose_name='vytvořeno dne'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='aktivní?'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='last_login',
            field=models.DateTimeField(blank=True, null=True,
                                       verbose_name='poslední přihlášení'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL, verbose_name='uživatel'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='uuid',
            field=models.CharField(max_length=32, unique=True,
                                   verbose_name='UUID'),
        ),
    ]



from datetime import timedelta

from django.conf import settings as django_settings
from django.contrib.auth.hashers import check_password, make_password
from django.contrib.auth.models import AnonymousUser
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.utils import IntegrityError
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from auth_token.config import settings
from auth_token.utils import generate_key


class Token(models.Model):
    """
    The default authorization token model.
    """

    TWO_FACTOR_CODE_LENGTH = 5
    TWO_FACTOR_CODE_SALT = 'two-factor'

    key = models.CharField(max_length=40, primary_key=True, null=False, blank=False)
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, related_name='auth_token', null=False, blank=False,
                             on_delete=models.CASCADE)
    created_at = models.DateTimeField(auto_now_add=True, null=False, blank=False)
    last_access = models.DateTimeField(auto_now=True, null=False, blank=False)
    is_active = models.BooleanField(default=True)
    # It is possiple use https://github.com/selwin/django-user_agents/tree/master/django_user_agents or
    # https://github.com/selwin/python-user-agents for parse
    # Limited size to 256
    user_agent = models.CharField(max_length=256, null=True, blank=True)
    expiration = models.BooleanField(default=True)
    ip = models.GenericIPAddressField(null=False, blank=False)
    auth_slug = models.SlugField(null=True, blank=True)
    backend = models.CharField(max_length=255, null=False, blank=False)
    allowed_cookie = models.BooleanField(default=True)
    allowed_header = models.BooleanField(default=True)
    is_authenticated = models.BooleanField(null=False, blank=False, default=False)
    two_factor_code = models.CharField(null=True, blank=True, max_length=128)

    is_from_header = False
    is_from_cookie = False

    @property
    def active_takeover(self):
        return self.user_takeovers.filter(is_active=True).last()

    def save(self, *args, **kwargs):
        if not self.key:
             self.key = self._generate_unique_key()
        return super().save(*args, **kwargs)

    def _generate_unique_key(self):
        """
        Generate random unique token key.
        """
        key = generate_key()
        try_generator_iterations = 1
        while self.__class__.objects.filter(key=key).exists():
            if try_generator_iterations > settings.MAX_RANDOM_KEY_ITERATIONS:
                raise IntegrityError('Could not produce unique key for authorization token')
            try_generator_iterations += 1
            key = generate_key()
        return key

    def _get_token_age(self):
        return self.expiration and settings.DEFAULT_TOKEN_AGE or settings.MAX_TOKEN_AGE

    @property
    def is_expired(self):
        return self.last_access + timedelta(seconds=self._get_token_age()) < timezone.now()

    @property
    def time_to_expiration(self):
        return (self.last_access + timedelta(seconds=self._get_token_age())) - timezone.now()

    @property
    def str_time_to_expiration(self):
        return str(self.time_to_expiration) if self.time_to_expiration.total_seconds() > 0 else '00:00:00'

    def __str__(self):
        return self.key


class TokenRelatedObject(models.Model):
    """
    Generic relation to objects related with authorization token
    """

    token = models.ForeignKey(Token, related_name='related_objects', on_delete=models.CASCADE)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.TextField()
    content_object = GenericForeignKey('content_type', 'object_id')


class UserTokenTakeover(models.Model):
    """
    The model allows to change user without token change
    """

    token = models.ForeignKey(Token, related_name='user_takeovers', on_delete=models.CASCADE)
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, related_name='user_token_takeovers', null=False,
                             blank=False, on_delete=models.CASCADE)
    is_active = models.BooleanField()


class AnonymousToken:

    key = None
    user = AnonymousUser
    creted_at = None
    is_active = False
    user_agent = None
    is_expired = True
    is_from_header = False
    is_from_cookie = False
    active_takeover = None
    backend = None
    allowed_cookie = False
    allowed_header = False

    def save(self):
        raise NotImplementedError

    def delete(self):
        raise NotImplementedError


class DeviceKeyAlreadyExistsException(Exception):
    pass


class DeviceKeyQuerySet(models.QuerySet):

    def create_token(self, uuid, user, user_agent=''):
        """
        This method must be called when user is authenticated.
        It creates a new DeviceKey with auto generated token for the device and returns token.
        If DeviceKey with same UUID exists DeviceKeyAlreadyExistsException is raised.
        """
        token = generate_key(length=64)
        is_created_new_device_key = self.get_or_create(
            uuid=uuid, user=user,
            defaults={
                'login_token': make_password(token),
                'user_agent': user_agent[:256],
                'is_active': True
            }
        )[1]
        if is_created_new_device_key:
            return token
        else:
            raise DeviceKeyAlreadyExistsException('Device key already exists')


class DeviceKey(models.Model):
    """Model used to authenticate mobile devices. Unhashed login_token is stored
    in the device keychain and serve as password to log in together with UUID via DeviceBackend."""

    created_at = models.DateTimeField(auto_now_add=True, null=False, blank=False, verbose_name=_('created at'))
    # this is not UUIDField because of the strict length limitation
    uuid = models.CharField(verbose_name=_('UUID'), max_length=32)
    last_login = models.DateTimeField(null=True, blank=True, verbose_name=_('last login'))
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, on_delete=models.CASCADE, verbose_name=_('user'))
    login_token = models.CharField(max_length=128, verbose_name=_('login token'))
    is_active = models.BooleanField(default=True, verbose_name=_('is active'))
    user_agent = models.CharField(max_length=256, null=True, blank=True, verbose_name=_('user agent'))

    class Meta:
        unique_together = ('uuid', 'user')
        verbose_name = _('device key')
        verbose_name_plural = _('device keys')

    objects = DeviceKeyQuerySet.as_manager()

    def __str__(self):
        return '{}, {}'.format(self.uuid, self.user)

    def check_password(self, token):
        return check_password(token, self.login_token)



from django.conf import settings as django_settings


DEFAULTS = {
    'COOKIE': True,  # Cookie authorization is allowed
    'COOKIE_NAME': 'Authorization',  # Authorization token cookie name
    'COOKIE_AGE': 60 * 60 * 24 * 7 * 2,  # Age of cookie, in seconds (default: 2 weeks)
    'COOKIE_HTTPONLY': False,  # Scripts can read authorization cookie
    'COOKIE_SECURE': False,  # Cookie can be sent only via HTTPS
    'COOKIE_DOMAIN': None,  # Cookie domain name
    'HEADER': True,  # Header authorization is allowed
    'HEADER_NAME': 'Authorization',  # Authorization token HTTP header name
    'HEADER_TOKEN_TYPE': 'Bearer',  # Type of the token
    'DEFAULT_TOKEN_AGE': 60 * 60,  # Default token expiration time (default: 1 hour)
    'MAX_TOKEN_AGE': 60 * 60 * 24 * 7 * 2,  # Max token expiration time (default: 2 weeks)
    'COUNT_USER_PRESERVED_TOKENS': 20,  # Maximum tokens that will be preserved per user
    'TAKEOVER_REDIRECT_URL': '/',  # The path where will be used after takeover another user token
    'TWO_FACTOR_ENABLED': False,  # Two factor authentication is disabled
    'TWO_FACTOR_REDIRECT_URL': 'login-code-verification/',  # The path the user is redirected to after successful two
                                                            # factor authentication
    'TWO_FACTOR_CODE_GENERATING_FUNCTION': 'auth_token.utils.generate_two_factor_code',  # Function, which generates
                                                                                         # code for two factor
                                                                                         # authentication
    'TWO_FACTOR_SENDING_FUNCTION': '',  # Function, which need to be implemented to send the key for second part of
                                        # authorization process to the user
    'TWO_FACTOR_DEBUG_TOKEN_SMS_CODE': None,  # Default code for two factor debug
    'RENEWAL_EXEMPT_HEADER': 'X-Authorization-Renewal-Exempt',  # Header name which causes that the token expiration
                                                                # time will not be extended
    'EXPIRATION_HEADER': 'X-Authorization-Expiration',  # Header name which contains information about token expiration
    'MAX_RANDOM_KEY_ITERATIONS': 100,  # Maximum iterations for random key generator
}


class Settings:

    def __getattr__(self, attr):
        if attr not in DEFAULTS:
            raise AttributeError('Invalid AUTH_TOKEN setting: "{}"'.format(attr))

        default = DEFAULTS[attr]
        return getattr(django_settings, 'AUTH_TOKEN_{}'.format(attr), default(self) if callable(default) else default)


settings = Settings()



from datetime import timedelta
from unittest.mock import patch

from django.contrib.auth.hashers import make_password
from django.test import override_settings
from django.utils import timezone
from django.utils.translation import ugettext as _
from nose.tools import assert_equal

from auth_token.config import settings
from auth_token.models import Token
from freezegun import freeze_time
from germanium.decorators import data_consumer
from germanium.test_cases.client import ClientTestCase
from germanium.test_cases.rest import RESTTestCase
from germanium.tools import assert_false, assert_in, assert_not_in, assert_true
from germanium.tools.http import (assert_http_accepted, assert_http_ok,
                                  assert_http_redirect)

from .base import BaseTestCaseMixin

__all__ = (
   'RESTLoginISCoreTestCase',
   'UILoginISCoreTestCase',
)


class RESTLoginISCoreTestCase(BaseTestCaseMixin, RESTTestCase):

    INDEX_URL = '/is_core/'
    API_LOGIN_URL = '/is_core/api/login/'

    @data_consumer('create_user')
    def test_user_should_be_authorized_via_http_header(self, user):
        assert_http_redirect(self.get(self.INDEX_URL))
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_in('token', resp.json())
        assert_http_ok(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        assert_not_in('Authorization', self.c.cookies)
        assert_true(Token.objects.last().allowed_header)
        assert_false(Token.objects.last().allowed_cookie)

    @override_settings(AUTH_TOKEN_HEADER=False)
    @data_consumer('create_user')
    def test_user_should_not_be_authorized_via_http_header_if_headers_are_turned_off(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_in('token', resp.json())
        assert_http_redirect(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        assert_false(self.client.cookies)

    @override_settings(AUTH_TOKEN_HEADER_NAME='X-Authorization')
    @data_consumer('create_user')
    def test_user_should_be_authorized_with_changed_header_name(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_in('token', resp.json())
        assert_http_ok(
            self.get(self.INDEX_URL, headers={'HTTP_X_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        assert_http_redirect(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )

    @data_consumer('create_user')
    def test_user_should_not_be_authorized_via_header_if_token_has_not_allowed_header(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_http_ok(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        Token.objects.all().update(allowed_header=False)
        assert_http_redirect(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )

    @override_settings(AUTH_TOKEN_HEADER_TOKEN_TYPE=None)
    @data_consumer('create_user')
    def test_user_should_be_authorized_via_header_without_token_type(self, user):

        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_http_redirect(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        assert_http_ok(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': resp.json()['token']})
        )

    @data_consumer('create_user')
    def test_token_type_should_be_required(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_http_ok(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        assert_http_redirect(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': resp.json()['token']})
        )

    @data_consumer('create_user')
    def test_token_should_be_updated_during_its_access(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        token = resp.json()['token']
        resp = self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        assert_in('X-Authorization-Expiration', resp)
        assert_true(resp['X-Authorization-Expiration'].startswith('0:59'))
        with freeze_time(timezone.now() + timedelta(minutes=10), tick=True):
            resp = self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
            assert_in('X-Authorization-Expiration', resp)
            assert_true(resp['X-Authorization-Expiration'].startswith('0:59'))

    @data_consumer('create_user')
    def test_token_should_not_be_updated_during_its_access_if_request_contains_specific_haeader(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        token = resp.json()['token']
        resp = self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        assert_in('X-Authorization-Expiration', resp)
        assert_true(resp['X-Authorization-Expiration'].startswith('0:59'))
        with freeze_time(timezone.now() + timedelta(minutes=10), tick=True):
            resp = self.get(self.INDEX_URL, headers={
                'HTTP_AUTHORIZATION': 'Bearer {}'.format(token),
                'HTTP_X_AUTHORIZATION_RENEWAL_EXEMPT': '1'
            })
            assert_in('X-Authorization-Expiration', resp)
            assert_true(resp['X-Authorization-Expiration'].startswith('0:49'))

    @data_consumer('create_user')
    def test_token_should_be_expired(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        token = resp.json()['token']
        resp = self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        assert_in('X-Authorization-Expiration', resp)
        assert_true(resp['X-Authorization-Expiration'].startswith('0:59'))
        with freeze_time(timezone.now() + timedelta(hours=1), tick=True):
            assert_http_redirect(self.get(self.INDEX_URL, headers={
                'HTTP_AUTHORIZATION': 'Bearer {}'.format(token),
            }))

    @data_consumer('create_user')
    def test_user_should_be_logged_out_via_http_header(self, user):
        resp = self.post(self.API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        token = resp.json()['token']
        assert_http_ok(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        )
        assert_http_accepted(
            self.delete(self.API_LOGIN_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        )
        assert_http_redirect(
            self.get(self.INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        )


class UILoginISCoreTestCase(BaseTestCaseMixin, ClientTestCase):

    INDEX_URL = '/is_core/'
    UI_LOGIN_URL = '/is_core/login/'
    UI_LOGOUT_URL = '/is_core/logout/'
    UI_TWO_FACTOR_LOGIN_URL = '/two-factor-login/'
    UI_CODE_CHECK_LOGIN_URL = '/login-code-verification/'
    CODE = '12345'

    def send_two_factor_token(token, code):
        pass

    def generate_code(self):
        return UILoginISCoreTestCase.CODE

    @data_consumer('create_user')
    def test_user_should_be_authorized_via_cookie(self, user):
        assert_http_redirect(self.get(self.INDEX_URL))
        resp = self.post(self.UI_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_redirect(resp)
        assert_http_ok(self.get(self.INDEX_URL))
        assert_in('Authorization', self.c.cookies)
        assert_false(Token.objects.last().allowed_header)
        assert_true(Token.objects.last().allowed_cookie)

    @override_settings(AUTH_TOKEN_COOKIE=False)
    @data_consumer('create_user')
    def test_user_should_not_be_authorized_via_cookie_if_cookie_is_turned_off(self, user):
        resp = self.post(self.UI_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_redirect(resp)
        assert_http_redirect(self.get(self.INDEX_URL))

    @override_settings(AUTH_TOKEN_COOKIE_NAME='ChangedAuthorization')
    @data_consumer('create_user')
    def test_user_should_be_authorized_with_changed_cookie_name(self, user):
        resp = self.post(self.UI_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_redirect(resp)
        assert_in('ChangedAuthorization', self.c.cookies)
        assert_http_ok(self.get(self.INDEX_URL))

    @data_consumer('create_user')
    def test_user_should_not_be_authorized_via_cookie_if_cookie_has_not_allowed_header(self, user):
        resp = self.post(self.UI_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_redirect(resp)
        assert_http_ok(self.get(self.INDEX_URL))
        assert_in('Authorization', self.c.cookies)
        Token.objects.all().update(allowed_cookie=False)
        assert_http_redirect(self.get(self.INDEX_URL))

    @data_consumer('create_user')
    def test_user_should_be_logged_out_via_cookies(self, user):
        resp = self.post(self.UI_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_redirect(resp)
        assert_http_ok(self.get(self.INDEX_URL))
        assert_http_ok(self.get(self.UI_LOGOUT_URL))
        assert_http_redirect(self.get(self.INDEX_URL))

    @override_settings(AUTH_TOKEN_TWO_FACTOR_ENABLED=True)
    @override_settings(
        AUTH_TOKEN_TWO_FACTOR_SENDING_FUNCTION='app.tests.is_core.UILoginISCoreTestCase.send_two_factor_token')
    @data_consumer('create_user')
    def test_user_should_be_authorized_with_two_factor_authentication(self, user):
        login_resp = self.post(self.UI_TWO_FACTOR_LOGIN_URL, {'username': 'test', 'password': 'test'})

        assert_http_redirect(login_resp)
        resp_location = login_resp['Location']
        assert_equal(resp_location[:resp_location.find('?')], settings.TWO_FACTOR_REDIRECT_URL)
        assert_false(login_resp.wsgi_request.user.is_authenticated)

        token = login_resp.wsgi_request.token
        assert_true(token.two_factor_code is not None)
        assert_false(token.is_authenticated)
        # the code value needs to be overwritted, so that its value could be used for next request
        token.two_factor_code = make_password('12345', salt=Token.TWO_FACTOR_CODE_SALT)
        token.save()

        code_check_resp = self.post(self.UI_CODE_CHECK_LOGIN_URL, {'code': '12345'})

        assert_http_redirect(code_check_resp)
        assert_equal(code_check_resp['location'], '/accounts/profile/')
        assert_true(code_check_resp.wsgi_request.token.is_authenticated)
        assert_true(code_check_resp.wsgi_request.user.is_authenticated)
        assert_equal(code_check_resp.wsgi_request.user, user)

    @override_settings(AUTH_TOKEN_TWO_FACTOR_ENABLED=True)
    @override_settings(
        AUTH_TOKEN_TWO_FACTOR_SENDING_FUNCTION='app.tests.is_core.UILoginISCoreTestCase.send_two_factor_token')
    @override_settings(
        AUTH_TOKEN_TWO_FACTOR_CODE_GENERATING_FUNCTION='app.tests.is_core.UILoginISCoreTestCase.generate_code')
    @patch('app.tests.is_core.UILoginISCoreTestCase.send_two_factor_token')
    def test_send_two_factor_token_should_be_called_for_two_factor_login(self, send_two_factor_token):
        self.create_user()
        login_resp = self.post(self.UI_TWO_FACTOR_LOGIN_URL, {'username': 'test', 'password': 'test'})

        assert_http_redirect(login_resp)
        send_two_factor_token.assert_called_once_with(login_resp.wsgi_request.token, self.CODE)

    @override_settings(AUTH_TOKEN_TWO_FACTOR_ENABLED=True)
    @override_settings(
        AUTH_TOKEN_TWO_FACTOR_SENDING_FUNCTION='app.tests.is_core.UILoginISCoreTestCase.send_two_factor_token')
    @data_consumer('create_user')
    def test_user_should_not_be_logged_in_two_factor_for_invalid_code(self, user):
        login_resp = self.post(self.UI_TWO_FACTOR_LOGIN_URL, {'username': 'test', 'password': 'test'})

        assert_http_redirect(login_resp)
        # the code value needs to be overwritten, so that its value could be used for next request
        login_resp.wsgi_request.token.two_factor_code = make_password('12345', salt=Token.TWO_FACTOR_CODE_SALT)
        login_resp.wsgi_request.token.save()

        code_check_resp = self.post(self.UI_CODE_CHECK_LOGIN_URL, {'code': 'other_code'})

        assert_http_ok(code_check_resp)
        assert_in(
            _('The inserted value does not correspond to the sent code.'),
            code_check_resp._container[0].decode('utf8')
        )
        assert_false(code_check_resp.wsgi_request.token.is_authenticated)



import time

from django.utils.encoding import force_text
from django.utils.functional import SimpleLazyObject
from django.utils.http import http_date

from auth_token import utils
from auth_token.config import settings
from auth_token.utils import header_name_to_django


def get_user(request):
    if not hasattr(request, '_cached_user'):
        request._cached_user = utils.get_user(request)
    return request._cached_user


class TokenAuthenticationMiddleware:

    def __init__(self, get_response=None):
        self.get_response = get_response
        super().__init__()

    def __call__(self, request):
        response = None
        if hasattr(self, 'process_request'):
            response = self.process_request(request)
        response = response or self.get_response(request)
        if hasattr(self, 'process_response'):
            response = self.process_response(request, response)
        return response

    def process_request(self, request):
        """
        Lazy set user and token
        """
        request.token = utils.get_token(request)
        request.user = SimpleLazyObject(lambda: get_user(request))
        request._dont_enforce_csrf_checks = utils.dont_enforce_csrf_checks(request)

    def _update_token_and_cookie(self, request, response, max_age, expires):
        request.token.save()
        if settings.COOKIE and request.token.allowed_cookie:
            response.set_cookie(settings.COOKIE_NAME, force_text(request.token.key), max_age=max_age,
                                expires=expires, httponly=settings.COOKIE_HTTPONLY,
                                secure=settings.COOKIE_SECURE, domain=settings.COOKIE_DOMAIN)
        return response

    def _set_auth_expiration_header(self, request, response):
        response[settings.EXPIRATION_HEADER] = request.token.str_time_to_expiration

    def _is_token_renewal_exempt(self, request):
        return (getattr(request, 'auth_token_renewal_exempt', False) or
                header_name_to_django(settings.RENEWAL_EXEMPT_HEADER) in request.META)

    def process_response(self, request, response):
        """
        Set cookie with token key if user is authenticated
        """
        # Save the session data and refresh the client cookie.
        # Skip session save for 500 responses, refs #3881.
        if response.status_code != 500 and hasattr(request, 'token') and request.token.is_active:
            if request.token.expiration:
                # The user did not choose to be permanently signed. Hence, the authentication cookie that holds the
                # token value is set to expire when the browser is closed
                max_age = None
                expires = None
            else:
                max_age = settings.COOKIE_AGE
                expires = http_date(time.time() + max_age)

            if not self._is_token_renewal_exempt(request):
                self._update_token_and_cookie(request, response, max_age, expires)

            self._set_auth_expiration_header(request, response)

        return response

    def process_view(self, request, view_func, view_args, view_kwargs):
        request.auth_token_renewal_exempt = getattr(view_func, 'auth_token_renewal_exempt', False)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-06-22 17:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0008_auto_20190509_1401'),
    ]

    operations = [
        migrations.AddField(
            model_name='token',
            name='is_authenticated',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='token',
            name='two_factor_code',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
    ]



from django.contrib.auth.views import LoginView

from auth_token.contrib.common.auth_security import LOGIN_THROTTLING_VALIDATORS
from auth_token.contrib.common.default.views import TokenLoginView as DefaultTokenLoginView

from security.decorators import throttling_all
from security.enums import InputLoggedRequestType


@throttling_all(*LOGIN_THROTTLING_VALIDATORS)
class TokenLoginView(DefaultTokenLoginView):

    def form_valid(self, form):
        if getattr(self.request, 'input_logged_request', False):
            self.request.input_logged_request.type = InputLoggedRequestType.SUCCESSFUL_LOGIN_REQUEST
        return super().form_valid(form)

    def form_invalid(self, form):
        if getattr(self.request, 'input_logged_request', False):
            self.request.input_logged_request.type = InputLoggedRequestType.UNSUCCESSFUL_LOGIN_REQUEST
        return super().form_invalid(form)


@throttling_all(*LOGIN_THROTTLING_VALIDATORS)
class LoginCodeVerificationView(LoginView):
    pass


class InputLogMixin:

    def log_successful_request(self):
        if getattr(self.request, 'input_logged_request', False):
            self.request.input_logged_request.type = InputLoggedRequestType.SUCCESSFUL_2FA_CODE_VERIFICATION_REQUEST

    def log_unsuccessful_request(self):
        if getattr(self.request, 'input_logged_request', False):
            self.request.input_logged_request.type = InputLoggedRequestType.UNSUCCESSFUL_2FA_CODE_VERIFICATION_REQUEST


# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-12 07:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth_token', '0005_auto_20180626_1724'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('uuid', models.UUIDField(unique=True)),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                ('login_token', models.CharField(max_length=128)),
                ('is_active', models.BooleanField(default=True)),
                ('user_agent',
                 models.CharField(blank=True, max_length=256, null=True)),
                ('user',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                   to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='devicekey',
            unique_together=set([('uuid', 'user')]),
        ),
    ]



# Generated by Django 2.0 on 2018-06-26 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0004_token_backend'),
    ]

    operations = [
        migrations.AddField(
            model_name='token',
            name='allowed_cookie',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='token',
            name='allowed_header',
            field=models.BooleanField(default=True),
        )
    ]



from germanium.decorators import data_consumer
from germanium.test_cases.rest import RESTTestCase
from germanium.tools.http import assert_http_ok, assert_http_unauthorized, assert_http_accepted, assert_http_bad_request
from germanium.tools import assert_true, assert_false, assert_in, assert_not_in, assert_equal

from auth_token.models import Token, DeviceKey

from .base import BaseTestCaseMixin


__all__ = (
    'RESTFrameworkLoginISCoreTestCase',
    'DeviceKeyTestCase'
)

UUID = 'E621E1F8C36C495A93FC0C247A3E6E5F'
SHORTER_UUID = 'E621E1F8C36C495'
INDEX_URL = '/api/'
API_LOGIN_URL = '/api/login/'


class RESTFrameworkLoginISCoreTestCase(BaseTestCaseMixin, RESTTestCase):

    API_LOGOUT_URL = '/api/logout/'

    @data_consumer('create_user')
    def test_user_should_be_authorized_via_http_header(self, user):
        assert_http_unauthorized(self.get(INDEX_URL))
        resp = self.post(API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        assert_in('token', resp.json())
        assert_http_ok(
            self.get(INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(resp.json()['token'])})
        )
        assert_not_in('Authorization', self.c.cookies)
        assert_true(Token.objects.last().allowed_header)
        assert_false(Token.objects.last().allowed_cookie)

    @data_consumer('create_user')
    def test_user_should_be_logged_out_via_http_header(self, user):
        resp = self.post(API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_ok(resp)
        token = resp.json()['token']
        assert_http_ok(
            self.get(INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        )
        assert_http_accepted(
            self.delete(self.API_LOGOUT_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        )
        assert_http_unauthorized(
            self.get(INDEX_URL, headers={'HTTP_AUTHORIZATION': 'Bearer {}'.format(token)})
        )


class DeviceKeyTestCase(BaseTestCaseMixin, RESTTestCase):

    API_REQUEST_MOBILE_TOKEN_URL = '/api/mobile-request-token/'
    API_MOBILE_LOGIN_URL = '/api/mobile-login/'

    @data_consumer('create_user')
    def test_user_should_be_authorized_from_token_and_uuid(self, user):
        device_token = DeviceKey.objects.create_token(uuid=UUID, user=user)
        resp = self.post(self.API_MOBILE_LOGIN_URL,
                         {'uuid': UUID, 'login_device_token': device_token})
        assert_http_ok(resp)
        assert_in('token', resp.json())
        assert_http_ok(
            self.get(INDEX_URL, headers={
                'HTTP_AUTHORIZATION': 'Bearer {}'.format(
                    resp.json()['token'])})
        )
        assert_not_in('Authorization', self.c.cookies)
        assert_true(Token.objects.last().allowed_header)
        assert_false(Token.objects.last().allowed_cookie)

    @data_consumer('create_user')
    def test_user_should_be_authorized_from_token_and_shorter_uuid(self, user):

        device_token = DeviceKey.objects.create_token(uuid=SHORTER_UUID, user=user)
        resp = self.post(self.API_MOBILE_LOGIN_URL,
                         {'uuid': SHORTER_UUID, 'login_device_token': device_token})
        assert_http_ok(resp)

    @data_consumer('create_user')
    def test_user_should_get_token_when_device_registered_by_uuid(self, user):
        logged_in_resp = self.post(API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        registration_mobile_resp = self.post(self.API_REQUEST_MOBILE_TOKEN_URL, headers={
            'HTTP_AUTHORIZATION': 'Bearer {}'.format(
                logged_in_resp.json()['token'])}, data={'uuid': UUID})
        assert_http_ok(registration_mobile_resp)
        mobile_token = registration_mobile_resp.json()['device_login_token']
        device_keys = DeviceKey.objects.all()
        assert_equal(1, device_keys.count())
        device_key = device_keys[0]
        assert_true(device_key.check_password(mobile_token))
        assert_false(device_key.check_password('XXX'))

    @data_consumer('create_user')
    def test_user_should_not_register_same_device_keys(self, user):
        logged_in_resp = self.post(API_LOGIN_URL, {'username': 'test', 'password': 'test'})
        registration_mobile_resp = self.post(self.API_REQUEST_MOBILE_TOKEN_URL, headers={
            'HTTP_AUTHORIZATION': 'Bearer {}'.format(
                logged_in_resp.json()['token'])}, data={'uuid': UUID})
        assert_http_ok(registration_mobile_resp)
        registration_mobile_resp = self.post(self.API_REQUEST_MOBILE_TOKEN_URL, headers={
            'HTTP_AUTHORIZATION': 'Bearer {}'.format(
                logged_in_resp.json()['token'])}, data={'uuid': UUID})
        assert_http_bad_request(registration_mobile_resp)

    def create_users(self):
        return [
            (i, self.create_user('test{}'.format(i), 'test{}@test.cz'.format(i), 'test{}'.format(i))) for i in range(5)
        ]

    @data_consumer('create_users')
    def test_different_users_should_register_and_authorize_with_same_device_keys(self, i, user):
        logged_in_resp = self.post(API_LOGIN_URL, {'username': 'test{}'.format(i), 'password': 'test{}'.format(i)})
        registration_mobile_resp = self.post(self.API_REQUEST_MOBILE_TOKEN_URL, headers={
            'HTTP_AUTHORIZATION': 'Bearer {}'.format(
                logged_in_resp.json()['token'])}, data={'uuid': UUID})
        assert_http_ok(registration_mobile_resp)
        registration_mobile_resp = self.post(self.API_REQUEST_MOBILE_TOKEN_URL, headers={
            'HTTP_AUTHORIZATION': 'Bearer {}'.format(
                logged_in_resp.json()['token'])}, data={'uuid': UUID})
        assert_http_bad_request(registration_mobile_resp)



from django.utils.translation import ugettext

from rest_framework.authentication import BaseAuthentication, CSRFCheck
from rest_framework import exceptions

from auth_token.config import settings
from auth_token.utils import get_token_key_from_request


class AuthTokenAuthentication(BaseAuthentication):

    def authenticate(self, request):
        """
        Returns a `User` if the request session currently has a logged in user.
        Otherwise returns `None`.
        """

        # Get the session-based user from the underlying HttpRequest object
        user = getattr(request._request, 'user', None)

        # Unauthenticated, CSRF validation not required
        if not user or not user.is_active:
            if get_token_key_from_request(request)[0] is not None:
                raise exceptions.AuthenticationFailed(ugettext('Invalid authorization token.'))
            return None

        self.enforce_csrf(request)

        # CSRF passed with authenticated user
        return (user, None)

    def enforce_csrf(self, request):
        """
        Enforce CSRF validation for session based authentication.
        """
        reason = CSRFCheck().process_view(request, None, (), {})
        if reason:
            # CSRF failed, bail with explicit error message
            raise exceptions.PermissionDenied('CSRF Failed: %s' % reason)

    def authenticate_header(self, request):
        return settings.HEADER_TOKEN_TYPE



from django.contrib.auth.models import User


class BaseTestCaseMixin:

    def create_user(self, username='test', email='test@test.cz', password='test'):
        return User.objects._create_user(username, email, password, is_staff=True, is_superuser=True)



from django.contrib.auth.backends import ModelBackend
from django.core.exceptions import PermissionDenied
from django.utils import timezone

from auth_token.models import DeviceKey


class DeviceBackend(ModelBackend):
    """
    Authenticates device with UUID and login_token (password)
    """

    def authenticate(self, request, device_uuid=None, mobile_login_token=None, **kwargs):
        if not device_uuid or not mobile_login_token:
            return None

        device_key_qs = DeviceKey.objects.filter(uuid=device_uuid, is_active=True)
        if not device_key_qs.exists():
            raise PermissionDenied(
                'DeviceKey with device_uuid "{}" not found.'.format(device_uuid))

        for device_key in device_key_qs:
            if device_key.check_password(mobile_login_token):
                user = self.get_user(device_key.user_id)
                if user:
                    device_key.last_login = timezone.now()
                    device_key.save()
                    return user
                else:
                    return None

        raise PermissionDenied('Provided invalid login_token to DeviceKey with device_uuid "{}".'.format(device_uuid))



from django import forms
from django.contrib.auth.hashers import make_password
from django.utils.translation import ugettext as _

from is_core.forms.forms import SmartForm

from auth_token.config import settings
from auth_token.contrib.common.forms import AuthenticationCleanMixin, TokenAuthenticationMixin
from auth_token.models import Token


class TokenAuthenticationSmartForm(TokenAuthenticationMixin, AuthenticationCleanMixin, SmartForm):
    pass


class LoginCodeVerificationForm(SmartForm):

    code = forms.CharField(max_length=40, required=True, label=_('code'))

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super().__init__(*args, **kwargs)

    def _init_code(self, field):
        if settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE:
            field.initial = settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE

    def get_user(self):
        return self.request.token.user

    def clean_code(self):
        code = self.cleaned_data.get('code')
        if (make_password(code, salt=Token.TWO_FACTOR_CODE_SALT) != self.request.token.two_factor_code
                and (not settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE or settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE != code)):
            raise forms.ValidationError(_('The inserted value does not correspond to the sent code.'))
        else:
            self.request.token.is_authenticated = True
            self.request.token.save()
        return code



from auth_token.contrib.common.default.views import TokenLogoutView

try:
    import security
    from auth_token.contrib.common.auth_security.views import InputLogMixin, TokenLoginView
    from auth_token.contrib.common.auth_security.views import LoginCodeVerificationView as _LoginCodeVerificationView
except ImportError:
    from auth_token.contrib.common.default.views import InputLogMixin, TokenLoginView
    from django.contrib.auth.views import LoginView as _LoginCodeVerificationView


class LoginView(TokenLoginView):
    pass


class LogoutView(TokenLogoutView):
    pass


class LoginCodeVerificationView(InputLogMixin, _LoginCodeVerificationView):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 14:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('auth_token', '0006_auto_20190312_0846'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='devicekey',
            options={'verbose_name': 'device key',
                     'verbose_name_plural': 'device keys'},
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True,
                                       verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='last_login',
            field=models.DateTimeField(blank=True, null=True,
                                       verbose_name='last login'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='login_token',
            field=models.CharField(max_length=128, verbose_name='login token'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user_agent',
            field=models.CharField(blank=True, max_length=256, null=True,
                                   verbose_name='user agent'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='uuid',
            field=models.UUIDField(unique=True, verbose_name='UUID'),
        ),
    ]



from germanium.decorators import data_consumer
from germanium.test_cases.client import ClientTestCase
from germanium.tools.http import assert_http_ok, assert_http_redirect
from germanium.tools import assert_true, assert_false, assert_in

from auth_token.models import Token

from .base import BaseTestCaseMixin


__all__ = (
   'AdminLoginISCoreTestCase',
)


class AdminLoginISCoreTestCase(BaseTestCaseMixin, ClientTestCase):

    INDEX_URL = '/admin/'
    LOGIN_URL = '/admin/login/'
    LOGOUT_URL = '/admin/logout/'

    @data_consumer('create_user')
    def test_user_should_log_and_logout_to_the_administration(self, user):
        assert_http_redirect(self.get(self.INDEX_URL))
        resp = self.post(self.LOGIN_URL, {'username': 'test', 'password': 'test'})
        assert_http_redirect(resp)
        assert_http_ok(self.get(self.INDEX_URL))
        assert_in('Authorization', self.c.cookies)
        assert_false(Token.objects.last().allowed_header)
        assert_true(Token.objects.last().allowed_cookie)
        assert_http_ok(self.get(self.LOGOUT_URL))
        assert_http_redirect(self.get(self.INDEX_URL))




from django.contrib.auth import authenticate
from django.utils.translation import ugettext_lazy as _

from auth_token.models import DeviceKey, DeviceKeyAlreadyExistsException

from rest_framework import serializers


class AuthTokenSerializer(serializers.Serializer):

    username = serializers.CharField(label=_('Username'))
    password = serializers.CharField(
        label=_('Password'),
        style={'input_type': 'password'},
        trim_whitespace=False
    )
    permanent = serializers.BooleanField(label=_('Permanent'), required=False)

    def validate(self, attrs):
        username = attrs.get('username')
        password = attrs.get('password')

        if username and password:
            user = authenticate(request=self.context.get('request'),
                                username=username, password=password)

            # The authenticate call simply returns None for is_active=False
            # users. (Assuming the default ModelBackend authentication
            # backend.)
            if not user:
                msg = _('Unable to log in with provided credentials.')
                raise serializers.ValidationError(msg, code='authorization')
        else:
            msg = _('Must include "username" and "password".')
            raise serializers.ValidationError(msg, code='authorization')

        attrs['user'] = user
        return attrs


class MobileAuthTokenSerializer(serializers.Serializer):

    uuid = serializers.CharField(label=_('device UUID'))
    login_device_token = serializers.CharField(
        label=_('Device Token'),
        style={'input_type': 'password'},
        trim_whitespace=False
    )

    def validate(self, attrs):
        uuid = attrs.get('uuid')
        token = attrs.get('login_device_token')

        if uuid and token:
            # DeviceBackend is called here
            user = authenticate(request=self.context.get('request'),
                                device_uuid=uuid, mobile_login_token=token)

            if not user:
                raise serializers.ValidationError(
                    _('Unable to log in with provided credentials.'),
                    code='authorization')
        else:
            raise serializers.ValidationError(
                _('Must include "uuid" and "token".'), code='authorization')

        attrs['user'] = user
        return attrs


class MobileAuthTokenRegisterSerializer(serializers.Serializer):

    uuid = serializers.CharField(label=_('device UUID'))

    def validate(self, attrs):
        uuid = attrs.get('uuid')
        try:
            attrs['token'] = DeviceKey.objects.create_token(uuid=uuid, user=self.context['request'].user)
        except DeviceKeyAlreadyExistsException:
            raise serializers.ValidationError(
                _('Device key was already registered.'), code='registration')
        return attrs



# Generated by Django 2.0 on 2019-07-23 12:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0009_auto_20190622_1920'),
    ]

    operations = [
        migrations.AlterField(
            model_name='devicekey',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='last_login',
            field=models.DateTimeField(blank=True, null=True, verbose_name='last login'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL,
                                    verbose_name='user'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='uuid',
            field=models.CharField(max_length=32, verbose_name='UUID'),
        ),
    ]



from django.conf import settings
from django.conf.urls import url, include
from django.contrib import admin

from app.resource import SimpleAPI
from auth_token.contrib.admin.views import LoginView as AdminLoginView
from auth_token.contrib.admin.views import LogoutView as AdminLogoutView
from auth_token.contrib.is_core_auth.views import LoginCodeVerificationView, TwoFactorLoginView
from auth_token.contrib.rest_framework_auth.views import LoginView as RESTFrameworkLoginView
from auth_token.contrib.rest_framework_auth.views import LogoutView as RESTFrameworkLogoutView
from auth_token.contrib.rest_framework_auth.views import (
    LoginAuthToken, LogoutAuthToken, MobileLoginAuthToken, MobileRegisterToken
)
from is_core.site import site as is_core_site

admin.site.login = AdminLoginView.as_view()
admin.site.logout = AdminLogoutView.as_view()

urlpatterns = [
    url('admin/', admin.site.urls),
    url('is_core/', include(is_core_site.urls)),
    url('two-factor-login/', TwoFactorLoginView.as_view()),
    url('login-code-verification/', LoginCodeVerificationView.as_view()),
    url('api/login/', LoginAuthToken.as_view()),
    url('api/logout/', LogoutAuthToken.as_view()),
    url('api/mobile-login/', MobileLoginAuthToken.as_view()),
    url('api/mobile-request-token/', MobileRegisterToken.as_view()),
    url('api/docs/login/', RESTFrameworkLoginView.as_view()),
    url('api/docs/logout/', RESTFrameworkLogoutView.as_view()),
    url('api/', SimpleAPI.as_view())
]


if settings.DEBUG:
    from django.conf.urls.static import static
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from auth_token.contrib.common.views import LoginView as _LoginView, LogoutView as _LogoutView


class LoginView(_LoginView):

    template_name = 'admin/login.html'


class LogoutView(_LogoutView):
    pass



from datetime import timedelta

from django.db.models import Count
from django.conf import settings as django_settings
from django.utils import timezone
from django.core.management.base import BaseCommand
from django.apps import apps

from auth_token.config import settings
from auth_token.models import Token


class Command(BaseCommand):

    def handle(self, **options):
        cleaned_users = apps.get_model(*django_settings.AUTH_USER_MODEL.split('.', 1)).objects.annotate(
            count_tokens=Count('auth_token')
        ).filter(count_tokens__gt=settings.COUNT_USER_PRESERVED_TOKENS)

        for user in cleaned_users:
            user_last_preserved_token_pks = Token.objects.filter(
                user=user
            ).order_by('-created_at')[:settings.COUNT_USER_PRESERVED_TOKENS].values('pk')
            removing_tokens_qs = Token.objects.filter(
                last_access__lt=timezone.now() - timedelta(seconds=settings.MAX_TOKEN_AGE),
                user=user
            ).exclude(pk__in=user_last_preserved_token_pks)
            self.stdout.write('Removing {} tokens of user {}'.format(removing_tokens_qs.count(), user))
            removing_tokens_qs.delete()



from auth_token.contrib.common.views import LoginView as _LoginView
from auth_token.contrib.common.views import LogoutView as _LogoutView
from auth_token.utils import login, logout
from rest_framework.compat import coreapi, coreschema
from rest_framework.response import Response
from rest_framework.schemas import ManualSchema
from rest_framework.views import APIView

from .serializers import (
    AuthTokenSerializer, MobileAuthTokenSerializer, MobileAuthTokenRegisterSerializer
)


class LoginAuthToken(APIView):

    throttle_classes = ()
    permission_classes = ()
    authentication_classes = ()
    serializer_class = AuthTokenSerializer
    if coreapi is not None and coreschema is not None:
        schema = ManualSchema(
            fields=[
                coreapi.Field(
                    name='username',
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title='Username',
                        description='Valid username for authentication',
                    ),
                ),
                coreapi.Field(
                    name='password',
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title='Password',
                        description='Valid password for authentication',
                    ),
                ),
                coreapi.Field(
                    name='permanent',
                    required=False,
                    location='form',
                    schema=coreschema.Boolean(
                        title='Permanent',
                        description='Define if login can expire',
                    ),
                ),
            ],
            encoding='application/json',
        )
    allowed_cookie = False
    allowed_header = True

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data['user']

        login(
            request._request, user, not serializer.validated_data.get('permanent', False),
            allowed_cookie=self.allowed_cookie, allowed_header=self.allowed_header
        )
        return Response({'token': request._request.token.key})


class MobileLoginAuthToken(APIView):

    throttle_classes = ()
    permission_classes = ()
    authentication_classes = ()
    serializer_class = MobileAuthTokenSerializer
    if coreapi is not None and coreschema is not None:
        schema = ManualSchema(
            fields=[
                coreapi.Field(
                    name='uuid',
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title='Device UUID',
                        description='Valid device UUID for authentication',
                    ),
                ),
                coreapi.Field(
                    name='login_device_token',
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title='Password',
                        description='Valid token for authentication',
                    ),
                ),
            ],
            encoding='application/json',
        )
    allowed_cookie = False
    allowed_header = True

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data['user']

        login(
            request._request, user,
            allowed_cookie=self.allowed_cookie, allowed_header=self.allowed_header
        )
        return Response({'token': request._request.token.key})


class LogoutAuthToken(APIView):

    def delete(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            logout(request._request)
        return Response(status=204)


class MobileRegisterToken(APIView):

    serializer_class = MobileAuthTokenRegisterSerializer
    if coreapi is not None and coreschema is not None:
        schema = ManualSchema(
            fields=[
                coreapi.Field(
                    name='uuid',
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title='Device UUID',
                        description='Valid device UUID for authentication',
                    ),
                ),
            ],
            encoding='application/json',
        )

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        serializer.is_valid(raise_exception=True)
        return Response({'device_login_token': serializer.validated_data['token']})


class LoginView(_LoginView):

    template_name = 'rest_framework_auth/login.html'


class LogoutView(_LogoutView):

    template_name = None

    def get_next_page(self):
        return super().get_next_page() or '/'



from django import forms
from django.contrib.auth import login as auth_login
from django.contrib.auth.hashers import make_password
from django.core.exceptions import ValidationError
from django.forms.utils import ErrorList
from django.http import HttpResponseRedirect
from django.views.generic.base import RedirectView
from django.urls import reverse

import import_string
from auth_token.config import settings
from auth_token.contrib.common.views import LoginView as _LoginView
from auth_token.contrib.common.views import LogoutView as _LogoutView
from auth_token.contrib.common.views import LoginCodeVerificationView as _LoginCodeVerificationView
from auth_token.contrib.is_core_auth.forms import LoginCodeVerificationForm
from auth_token.models import Token
from auth_token.utils import login, takeover

from is_core.generic_views import DefaultCoreViewMixin
from is_core.generic_views.mixins import GetCoreObjViewMixin


class LoginView(_LoginView):

    template_name = 'is_core/login.html'


class TwoFactorLoginView(LoginView):

    def _generate_and_send_two_factor_code(self):
        code = import_string(settings.TWO_FACTOR_CODE_GENERATING_FUNCTION)(Token.TWO_FACTOR_CODE_LENGTH)
        import_string(settings.TWO_FACTOR_SENDING_FUNCTION)(self.request.token, code)
        self.request.token.two_factor_code = make_password(code, salt=Token.TWO_FACTOR_CODE_SALT)
        self.request.token.save()

    def _login(self, user, expiration, form):
        login(
            self.request, user, expiration, allowed_cookie=self.allowed_cookie, allowed_header=self.allowed_header,
            two_factor_login=True
        )

    def get_success_url(self):
        return '{url}?{redirect_field_name}={value}'.format(
            url=settings.TWO_FACTOR_REDIRECT_URL, redirect_field_name=self.redirect_field_name,
            value=self.get_redirect_url()
        )

    def form_valid(self, form):
        """
        The user has provided valid credentials (this was checked in AuthenticationForm.is_valid()). So now we
        can check the test cookie stuff and log him in.
        """
        self._login(form.get_user(), not form.is_permanent(), form)
        try:
            # send user the code for second part of authentication process
            self._generate_and_send_two_factor_code()
        except ValidationError as err:
            form._errors[forms.forms.NON_FIELD_ERRORS] = ErrorList([err])
            return self.form_invalid(form)
        return HttpResponseRedirect(self.get_success_url())


class LogoutView(_LogoutView):

    template_name = 'is_core/logged_out.html'


class UserTakeover(GetCoreObjViewMixin, DefaultCoreViewMixin, RedirectView):

    def get_redirect_url(self, *args, **kwargs):
        return settings.TAKEOVER_REDIRECT_URL

    def get(self, request, *args, **kwargs):
        user = self.get_obj()
        takeover(request, user)
        return super().get(request, *args, **kwargs)


class LoginCodeVerificationView(_LoginCodeVerificationView):

    template_name = 'is_core/login.html'
    form_class = LoginCodeVerificationForm

    def form_valid(self, form):
        self.log_successful_request()
        auth_login(self.request, form.get_user(), self.request.token.backend)
        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form):
        self.log_unsuccessful_request()
        return super().form_invalid(form)

    def dispatch(self, request, *args, **kwargs):
        if self.request.token.is_active:
            return super().dispatch(request, *args, **kwargs)
        else:
            return HttpResponseRedirect('{url}?{redirect_field_name}={value}'.format(
                url=reverse('IS:login', ), redirect_field_name=self.redirect_field_name,
                value=self.get_redirect_url()
            ))
